#include "MismatchQuery.h"

void MismatchQuery::firstStep()
{
    // Number of pieces to be created
    pcount = klimit;
    if (pcount <= 1)
        ++pcount;

    piecek = new int[pcount];
    psize = patlen%pcount == 0 ? patlen/pcount : patlen/pcount + 1;

    assert(!reversed);
    assert(!p->isReversed());
    for (unsigned maxk = 0; maxk <= klimit; ++maxk)
    {
        for (unsigned i = 0; i < pcount; ++i) 
            piecek[i] = 0;

        distribute(0, maxk, maxk);
        if (sum >= report) break;
        if ((callcounter - counterstart) > rlimit) break;
    }
    delete [] piecek;
}


void MismatchQuery::distribute(unsigned offset, int maxk, int k)
{
    if (sum >= report) return;
    
    if (k == 0)
    {
        if (debug)
        {
            for (unsigned i = 0; i < pcount; ++i)
                std::cerr << piecek[i] << " ";
            std::cerr << std::endl;
        }

        if (useForwardIndex())
            firstPiece();
        else
        {
            swap();        // Use reverse index
            p->reverse();  // and reverse pattern
            pat = p->c_str();
            revarr<int>(piecek, pcount);

            firstPiece();  // Search

            swap();        // Revert to forward index
            p->reverse();  // and restore pattern
            pat = p->c_str();
            revarr<int>(piecek, pcount);
            assert(!p->isReversed());
            assert(!reversed);
        }
        return;
    }

    for (unsigned i = offset; i < pcount; ++i)
    {
        ++piecek[i];
        distribute(i, maxk, k-1);
        --piecek[i];

        if (sum >= report) return;
        if (callcounter - counterstart > rlimit) return;
    }
}


void MismatchQuery::firstPiece()
{
    match.clear();
    while (!smin.empty()) smin.pop();
    while (!smax.empty()) smax.pop();
    
    smin.push(0);
    smax.push(textlen-1);

    nextPiece(patlen, ((patlen-1)/psize)*psize, 0);
}

void MismatchQuery::nextPiece(int pos, int border, int k)
{
    callcounter++;
//    if (callcounter - counterstart > rlimit) return;
//    if (sum >= report) return;

    if (k == piecek[border/psize]) 
    {
        exactMatch(pos, border);
        return;
    }

    if (pos == border) 
        return;

    // ALPHABET has been defined in BTSearch
    for (const char *c = ALPHABET; c < ALPHABET + ALPHABET_SIZE; ++c) {
	if (!pushChar(*c)) continue;
        if (pat[pos - 1] == *c) nextPiece(pos - 1, border, k);
        else nextPiece(pos - 1, border, k + 1);

        if (sum >= report) return;
        if (callcounter - counterstart > rlimit) return;
	popChar();
    }
}

void MismatchQuery::exactMatch(int pos, int border) {
    callcounter++;
    if (sum >= report) return;
        
    int i = pos;
    while (i != border && pushChar(pat[i-1])) 
        --i;

    if (i == 0)
        newMatch();
    else
        if (i == border)
            nextPiece(i, border-psize, 0);

    // Rewind stack
    while (i != pos) {
        ++i;
	popChar();
    }
}
